@@ -22,8 +22,8 @@ import numbers
 import os
 import pathlib
 import platform
+import portalocker
 import re
-import rocket3
 import signal
 import sys
 import threading
@@ -99,6 +99,8 @@ DEFAULTS = dict(
 
 HELPERS = {name: getattr(yatl.helpers, name) for name in yatl.helpers.__all__}
 
+_DEFAULT_APP_ROOTS = set()
+
 ART = r"""
 ██████╗ ██╗   ██╗██╗  ██╗██╗    ██╗███████╗██████╗
 ██╔══██╗╚██╗ ██╔╝██║  ██║██║    ██║██╔════╝██╔══██╗
@@ -168,6 +170,7 @@ def safely(func, exceptions=(Exception,), log=False, default=None):
 # Implement a O(1) LRU cache and memoize with expiration and monitoring (using linked list)
 #########################################################################################
 
+
 class Node:
     def __init__(self, key=None, value=None, t=None, m=None, prev=None, next=None):
         self.key, self.value, self.t, self.m, self.prev, self.next = (
@@ -376,6 +379,9 @@ def thread_safe_pydal_patch():
         "readable",
         "writable",
         "default",
+        "filter_in",
+        "filter_out",
+        "label",
         "update",
         "requires",
         "widget",
@@ -906,7 +912,7 @@ class action:
             except HTTP as http:
                 response.status = http.status
                 ret = getattr(http, "body", "")
-                http_headers = getattr(http, 'headers', None)
+                http_headers = getattr(http, "headers", None)
                 if http_headers:
                     response.headers.update(http_headers)
                 return ret
@@ -930,13 +936,20 @@ class action:
     def __call__(self, func):
         """Building the decorator"""
         app_name = action.app_name
-        base_path = "" if app_name == "_default" else "/%s" % app_name
-        path = (base_path + "/" + self.path).rstrip("/")
+        if self.path[0] == "/":
+            path = self.path.rstrip("/")
+        else:
+            if app_name == "_default":
+                base_path = ""
+                _DEFAULT_APP_ROOTS.add(self.path.split("/", 1)[0])
+            else:
+                base_path = f"/{app_name}"
+            path = (f"{base_path}/{self.path}").rstrip("/")
         if func not in self.registered:
             func = action.catch_errors(app_name, func)
         func = bottle.route(path, **self.kwargs)(func)
         if path.endswith("/index"):  # /index is always optional
-            func = bottle.route(path[:-6] or '/', **self.kwargs)(func)
+            func = bottle.route(path[:-6] or "/", **self.kwargs)(func)
         self.registered.add(func)
         return func
 
@@ -993,6 +1006,18 @@ def new_sslwrap(
 def get_error_snapshot(depth=5):
     """Return a dict describing a given traceback (based on cgitb.text)."""
 
+    tb = traceback.format_exc()
+    logfile = os.environ.get("PY4WEB_LOGFILE")
+    if logfile:
+        msg = f"[{datetime.datetime.now().isoformat()}]: {tb}\n"
+        if logfile == ':stderr':
+            sys.stderr.write(msg)
+        elif logfile == ':stdout':
+            sys.stdout.write(msg)
+        else:            
+            with portalocker.Lock(logfile, "a", timeout=2) as fp:
+                fp.write(msg)
+
     etype, evalue, etb = sys.exc_info()
     if isinstance(etype, type):
         etype = etype.__name__
@@ -1020,7 +1045,7 @@ def get_error_snapshot(depth=5):
 
     data["platform_info"] = {key: getattr(platform, key)() for key in platform_keys}
     data["os_environ"] = {key: str(value) for key, value in os.environ.items()}
-    data["traceback"] = traceback.format_exc()
+    data["traceback"] = tb
     data["exception_type"] = str(etype)
     data["exception_value"] = str(evalue)
 
@@ -1185,23 +1210,32 @@ class Reloader:
         # used by watcher
         def hook(*a, **k):
             app_name = request.path.split("/")[1]
-            if app_name in DIRTY_APPS:
+            if not app_name or app_name in _DEFAULT_APP_ROOTS:
+                app_name = "_default"
+            if DIRTY_APPS.get(app_name):
                 Reloader.import_app(app_name)
-                del DIRTY_APPS[app_name]
+                DIRTY_APPS[app_name] = False
             ## APP_WATCH tasks, if used by any app
             try_app_watch_tasks()
 
         _REQUEST_HOOKS.before.add(hook)
 
     @staticmethod
-    def clear_routes(app_name=''):
-        if app_name and app_name[0] != '/':
-            app_name = '/' + app_name
-        routes = app_name + '/*'
-        app = bottle.default_app()
-        app.router.remove(routes)
+    def clear_routes(app_name=""):
+        app_root = app_name
+        if app_root and app_root[0] != "/":
+            app_root = f"/{app_root}"
+        routes = f"{app_root}/*"
+        remove_route = bottle.default_app().router.remove
+        remove_route(routes)
         if app_name:
-            app.router.remove(app_name)
+            remove_route(app_root)
+            if app_name == '_default':
+                [(remove_route(f"/{_}"), remove_route(f"/{_}/*")) for _ in _DEFAULT_APP_ROOTS]
+                remove_route("/")
+                _DEFAULT_APP_ROOTS.clear()
+        else:
+            _DEFAULT_APP_ROOTS.clear()
 
     @staticmethod
     def import_apps():
@@ -1279,7 +1313,11 @@ class Reloader:
 
         if os.path.exists(static_folder):
             app_name = path.split(os.path.sep)[-1]
-            prefix = "" if app_name == "_default" else ("/%s" % app_name)
+            if app_name == "_default":
+                prefix = ""
+                _DEFAULT_APP_ROOTS.add('static')
+            else:
+                prefix = f"/{app_name}"
 
             @bottle.route(prefix + r"/static/<re((_\d+(\.\d+){2}/)?)><fp.path()>")
             def server_static(fp, static_folder=static_folder):
@@ -1314,10 +1352,10 @@ class Reloader:
 
 ERROR_PAGES = {
     "*": (
-        '<html><head><style>body{color:white;text-align: center;background-color:[[=color]];font-family:serif} '
-        'h1{font-size:6em;margin:16vh 0 8vh 0} h2{font-size:2em;margin:8vh 0} '
-        'a{color:white;text-decoration:none;font-weight:bold;padding:10px 10px;border-radius:10px;border:2px solid #fff;transition: all .5s ease} '
-        'a:hover{background:rgba(0,0,0,0.1);padding:10px 30px}</style></head>'
+        "<html><head><style>body{color:white;text-align: center;background-color:[[=color]];font-family:serif} "
+        "h1{font-size:6em;margin:16vh 0 8vh 0} h2{font-size:2em;margin:8vh 0} "
+        "a{color:white;text-decoration:none;font-weight:bold;padding:10px 10px;border-radius:10px;border:2px solid #fff;transition: all .5s ease} "
+        "a:hover{background:rgba(0,0,0,0.1);padding:10px 30px}</style></head>"
         '<body><h1>[[=code]]</h1><h2>[[=message]]</h2>[[if button_text:]]<a href="[[=href]]">[[=button_text]]</a>[[pass]]</body></html>'
     ),
 }
@@ -1493,16 +1531,15 @@ def start_server(kwargs):
             server_config["server"] = "gunicorn"
 
     # Catch interrupts like Ctrl-C if needed
-    if server_config["server"] not in ("rocket", "Twisted"):
-
+    if server_config["server"] not in {"rocket", "wsgirefWsTwistedServer"}:
         signal.signal(
             signal.SIGINT,
             lambda signal, frame: click.echo(
                 "KeyboardInterrupt (ID: {}) has been caught. Cleaning up...".format(
                     signal
                 )
-            )
-            and sys.exit(0),
+                and sys.exit(0),
+            ),
         )
 
     params["server"] = server_config["server"]
@@ -1855,6 +1892,7 @@ def new_app(apps_folder, app_name, yes, scaffold_zip):
     "--ssl_cert", type=click.Path(exists=True), help="SSL certificate file for HTTPS"
 )
 @click.option("--ssl_key", type=click.Path(exists=True), help="SSL key file for HTTPS")
+@click.option("--logfile", default="", help="Where to send error logs (:stdout/:stderr/{filename})")
 def run(**kwargs):
     """Run all the applications on apps_folder"""
     install_args(kwargs)
