@@ -19,7 +19,6 @@ class SSO(object):
 
     def __init__(self, **parameters):
         self.parameters = parameters
-        self.next = URL('index') # Destination after login succeeds
 
     def get_login_url(self):
         """returns the url for login"""
@@ -27,7 +26,7 @@ class SSO(object):
 
     def handle_request(self, auth, path, get_vars, post_vars):
         if path == "login":
-            self.next = request.query.get('next') or URL('index')
+            auth.session["_next"] = request.query.get("next") or URL("index")
             redirect(self.get_login_url())
         elif path == "callback":
             self._handle_callback(auth, get_vars)
@@ -70,7 +69,12 @@ class SSO(object):
                 data["id"] = data.get("username") or data.get("email")
         user_id = data.get("id")
         auth.store_user_in_session(user_id)
-        redirect(self.next)
+        if "_next" in auth.session:
+            next = auth.session.get("_next")
+            del auth.session["_next"]
+        else:
+            next = URL("index")
+        redirect(next)
 
     @staticmethod
     def _build_url(base, data):
@@ -164,10 +168,13 @@ class OAuth2(SSO):
             # Lets not get the  user attributes via the userinfo endpoint
             # but lets take the userinfo directly extracted from the token
             # res = requests.get(self.userinfo_url, headers=headers)
-            data = jwt.decode(token, algorithms=self.algorithms,
-                              # because of this open issue
-                              # https://github.com/jpadilla/pyjwt/issues/359
-                              options={"verify_signature": False})
+            data = jwt.decode(
+                token,
+                algorithms=self.algorithms,
+                # because of this open issue
+                # https://github.com/jpadilla/pyjwt/issues/359
+                options={"verify_signature": False},
+            )
         else:
             # fallback to old approach if "id_token" is not in the response
             token = output.get("access_token")
